/*
 * Copyright (c) 2018 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-BSD-5-Clause-Nordic
 */

#include <zephyr.h>
#include <rsa.h>
#include <mbedtls/platform.h>
#include <mbedtls/entropy_poll.h>
#include <bignum.h>
#include <random/rand32.h>
#include <string.h>

int f_rng(void *ctx, unsigned char *buf, size_t len)
{
    int e;
    //e = mbedtls_hardware_poll(NULL, buf, len, NULL);
    e = sys_csrand_get(buf, len);
    return e;
}

void main(void)
{
	int ret;

	printk("Hello, World!\n");

	mbedtls_rsa_context ctx;

	mbedtls_mpi P;
	mbedtls_mpi Q;
	mbedtls_mpi E;

	mbedtls_mpi_init(&P);
	mbedtls_mpi_init(&Q);
	mbedtls_mpi_init(&E);
	ret = mbedtls_mpi_read_string(&P, 16, "df7ce3bac471203ee36f9e3753e52a6d");
    //ret = mbedtls_mpi_lset(&P, 3);
	if (ret != 0) {
		printk("Failed to initialize P: %d\n", ret);
	}
	ret = mbedtls_mpi_read_string(&Q, 16, "8f1637fb056229e4895df4b07dfbae1d"); 
    //ret = mbedtls_mpi_lset(&Q, 5);
	if (ret != 0) {
		printk("Failed to initialize Q: %d\n", ret);
	}
    ret = mbedtls_mpi_read_string(&E, 16, "10001");
	//ret = mbedtls_mpi_lset(&E, 9);
	if (ret != 0) {
		printk("Failed to initialize E: %d\n", ret);
	}

	mbedtls_rsa_init(&ctx, MBEDTLS_RSA_PKCS_V15, 0);
	ret = mbedtls_rsa_import(&ctx, NULL, &P, &Q, NULL, &E);
	if (ret != 0) {
		printk(" failed\n  ! mbedtls_rsa_inport returned %d\n\n", ret);
		return;
	}
	if ((ret = mbedtls_rsa_complete(&ctx)) != 0) {
		printk(" failed\n  ! mbedtls_rsa_complete returned %d\n\n", ret);
		return;
	}

    unsigned char buf[1024] = {0};
    char *secret = "secret";
    ret = mbedtls_rsa_rsaes_pkcs1_v15_encrypt(&ctx, f_rng, NULL, MBEDTLS_RSA_PUBLIC, strlen(secret), secret, buf);
    if (ret != 0) {
        printk("mbedtls_rsa_rsaes_pkcs1_v15_encrypt failed: %d\n", ret);
        return;
    }

    printk("SUCCESS!\n");
    printk("%s\n", buf);
	
}
